#include <iostream>
#include <chrono>
#include <thread>
#include <iomanip>
#include "lookup.hpp"
#include "kinematics.hpp"
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif
#ifndef M_PI_2
#define M_PI_2 1.57079632679489661923
#endif

using namespace hebi;

int main()
{
  // Get a group
  Lookup lookup;
  std::unique_ptr<Group> group = lookup.getGroupFromNames({ "base", "shoulder", "elbow" }, { "family" });

  if (!group)
  {
    std::cout << "Group not found!";
    return -1;
  }

  // Hardcode the kinematics of a fixed corner bracket
  Eigen::Vector3f bracket_com;
  bracket_com << 0, 0.01, 0.01;

  Eigen::Matrix4f bracket_out = Matrix4f::Identity();
  bracket_out.topLeftCorner<3,3>() = AngleAxisf(M_PI/2, Eigen::Vector3f::UnitX()).matrix();

  Eigen::Vector3f bracket_translate;
  bracket_translate << 0.0, -0.045, 0.045;
  bracket_out.topRightCorner<3,1>() = bracket_translate;
  

  // Create a simple kinematic description of the arm 
  kinematics::Kinematics kin;
  kin.addBody(kinematics::KinematicBody::createX5());
  kin.addBody(kinematics::KinematicBody::createGenericLink(bracket_com, bracket_out));
  kin.addBody(kinematics::KinematicBody::createX5());
  kin.addBody(kinematics::KinematicBody::createX5Link(0.18, M_PI));
  kin.addBody(kinematics::KinematicBody::createX5());
  kin.addBody(kinematics::KinematicBody::createX5Link(0.28, 0));


  // Add a callback function to print (x,y,z) position
  Eigen::Matrix4f transform;
  group->addFeedbackHandler([&kin, &transform](const GroupFeedback* group_fbk)
  {
    Eigen::VectorXd angles = group_fbk->getPosition();
    kin.getEndEffector(FrameTypeOutput, angles, transform);
    std::cout << std::setw(0.2) <<
       "x " << transform(0,3) <<
      " y " << transform(1,3) <<
      " z " << transform(2,3) << std::endl;
  });
  
  // Control the robot at 100 Hz for 30 seconds
  group->setFeedbackFrequencyHz(20);
  std::this_thread::sleep_for(std::chrono::seconds(30));
  group->clearFeedbackHandlers();

  return 0;
}
