#include <iostream>
#include <chrono>
#include <thread>
#include <iomanip>
#include "lookup.hpp"
#include "kinematics.hpp"
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif
#ifndef M_PI_2
#define M_PI_2 1.57079632679489661923
#endif

using namespace hebi;

int main()
{
  // Get a group
  Lookup lookup;
  std::unique_ptr<Group> group = lookup.getGroupFromNames({ "base", "shoulder", "elbow" }, { "family" });

  if (!group)
  {
    std::cout << "Group not found!";
    return -1;
  }

  Eigen::Vector3f bracket_com;
  bracket_com << 0, 0, 0;

  Eigen::Matrix4f bracket_out = Matrix4f::Identity();
  bracket_out.topLeftCorner<3,3>() = AngleAxisf(M_PI/2, Eigen::Vector3f::UnitX()).matrix();

  Eigen::Vector3f bracket_translate;
  bracket_translate << 0.0, 0.045, 0.045;
  bracket_out.topRightCorner<3,1>() = bracket_translate;

  // Create a simple kinematic description of the arm 
  kinematics::Kinematics kin;
  kin.addBody(kinematics::KinematicBody::createX5());
  kin.addBody(kinematics::KinematicBody::createGenericLink(bracket_com, bracket_out));
  kin.addBody(kinematics::KinematicBody::createX5());
  kin.addBody(kinematics::KinematicBody::createX5Link(0.18, M_PI));
  kin.addBody(kinematics::KinematicBody::createX5());
  kin.addBody(kinematics::KinematicBody::createX5Link(0.28, 0));

  Eigen::Vector3f target_xyz;
  target_xyz << 0.4, 0.0, 0.2;
  Eigen::VectorXd start_joints(group->size());
  Eigen::VectorXd target_joints(group->size());

  // Get feedback
  GroupFeedback group_fbk(group->size());
  if (!group->requestFeedback(&group_fbk))
  {
    std::cout << "Couldn't get feedback!";
    return -1;
  }

  for (int i = 0; i < group_fbk.size(); ++i)
  {
    // Note -- should check whether this is valid.
    start_joints(i) = group_fbk[i].actuator().position().get();
  }

  kin.solveIK(target_xyz, start_joints, target_joints);

  // Set the command lifetime
  group->setCommandLifetimeMs(100);

  // Move the arm (note -- could use the Hebi Trajectory API to do this smoothly)
  GroupCommand group_cmd(group->size());
  group_cmd.setPosition(target_joints);

  // Note -- the arm will go limp after the 100 ms command lifetime, so we repeat
  // the command in a loop here until we terminate after approximately 5 seconds.
  for (int i = 0; i < 100; ++i)
  {
    group->sendCommand(group_cmd);
    std::this_thread::sleep_for(std::chrono::milliseconds(50));
  }

  return 0;
}
